using Benday.TfsUtility.JsonMessageTypes;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using Newtonsoft.Json.Linq;
namespace Benday.TfsUtility
{
    public class BuildDefinitionImportCommand : TfsCommandBase
    {
        public BuildDefinitionImportCommand(string[] args)
            : base(args)
        {

        }

        protected override List<string> GetRequiredArguments()
        {
            var argumentNames = new List<string>();

            argumentNames.Add(TfsUtilityConstants.ArgumentNameTfsCollection);
            argumentNames.Add(TfsUtilityConstants.ArgumentNameTeamProject);
            argumentNames.Add(TfsUtilityConstants.ArgumentNameReleaseDefinitionName);
            argumentNames.Add(TfsUtilityConstants.ArgumentNameFilename);

            return argumentNames;
        }

        protected override void DisplayUsage(StringBuilder builder)
        {
            base.DisplayUsage(builder);

            string usageString =
                String.Format("{0} {1} /collection:collectionurl /project:projectname /{2}:buildname",
                TfsUtilityConstants.ExeName,
                CommandArgumentName,
                TfsUtilityConstants.ArgumentNameBuildDefinitionName);

            builder.AppendLine(usageString);
        }

        protected override string CommandArgumentName
        {
            get { return TfsUtilityConstants.CommandArgumentNameBuildDefinitionImport; }
        }

        private BuildDefinitionInfo GetBuildInfoByBuildName(string buildDefinitionName)
        {
            string template = "{0}/{1}/_apis/build/definitions?api-version={2}&name={3}";
            // https://{instance}/defaultcollection/{project}/_apis/build/definitions?api-version={version}[&name={string}][&type={string}]

            string requestUrl = String.Format(
                template,
                Arguments[TfsUtilityConstants.ArgumentNameTfsCollection],
                Arguments[TfsUtilityConstants.ArgumentNameTeamProject],
                "2.0",
                buildDefinitionName);

            using (WebClient client = new WebClient())
            {
                client.UseDefaultCredentials = true;

                string result = client.DownloadString(requestUrl);

                var results =
                    JsonConvert.DeserializeObject<BuildDefinitionInfoResults>(result);

                if (results == null || results.Count == 0)
                {
                    return null;
                }
                else
                {
                    return results.Values[0];
                }
            }
        }

        private void UploadUpdatedBuildDefinition(string buildId, string revision, string buildDefinitionData)
        {
            string template = "{0}/{1}/_apis/build/definitions/{2}?api-version={3}";

            string requestUrl = String.Format(
                template,
                Arguments[TfsUtilityConstants.ArgumentNameTfsCollection],
                Arguments[TfsUtilityConstants.ArgumentNameTeamProject],
                buildId, 
                "2.0");

            using (WebClient client = new WebClient())
            {
                client.UseDefaultCredentials = true;
                client.Headers.Add("Content-Type", "application/json");

                var definition = new BuildDefinitionForUploadModified();

                definition.Name = Arguments[TfsUtilityConstants.ArgumentNameBuildDefinitionName];
                var buildDefinitionDataAsJson = JObject.Parse(buildDefinitionData);

                definition.Id = buildId;
                definition.Revision = revision;
                definition.Build = buildDefinitionDataAsJson["build"];
                definition.Repository = buildDefinitionDataAsJson["repository"];
                definition.Options = buildDefinitionDataAsJson["options"];
                definition.Variables = buildDefinitionDataAsJson["variables"];
                definition.Triggers = buildDefinitionDataAsJson["triggers"];
                definition.Comment = String.Empty;

                string buildDefinitionUploadAsJson = JsonConvert.SerializeObject(definition, Formatting.Indented);

                string result = client. UploadString(
                    requestUrl, "PUT", buildDefinitionUploadAsJson);
            }
        }

        private void UploadNewBuild(string buildDefinitionData)
        {
            string template = "{0}/{1}/_apis/build/definitions?api-version={2}";

            string requestUrl = String.Format(
                template,
                Arguments[TfsUtilityConstants.ArgumentNameTfsCollection],
                Arguments[TfsUtilityConstants.ArgumentNameTeamProject],
                "2.0");

            using (WebClient client = new WebClient())
            {
                client.UseDefaultCredentials = true;
                client.Headers.Add("Content-Type", "application/json");

                var definition = new BuildDefinitionForUploadNew();

                definition.Name = Arguments[TfsUtilityConstants.ArgumentNameBuildDefinitionName];
                var buildDefinitionDataAsJson = JObject.Parse(buildDefinitionData);

                definition.Build = buildDefinitionDataAsJson["build"];
                definition.Repository = buildDefinitionDataAsJson["repository"];
                definition.Options = buildDefinitionDataAsJson["options"];
                definition.Variables = buildDefinitionDataAsJson["variables"];
                definition.Triggers = buildDefinitionDataAsJson["triggers"];
                definition.Comment = String.Empty;

                string buildDefinitionUploadAsJson = JsonConvert.SerializeObject(definition, Formatting.Indented);

                string result = client.UploadString(requestUrl, buildDefinitionUploadAsJson);
            }
        }

        public void Upload()
        {
            string filenameArgValue = Arguments[TfsUtilityConstants.ArgumentNameFilename];

            var filename = Path.Combine(Environment.CurrentDirectory, filenameArgValue);

            if (File.Exists(filename) == false)
            {
                throw new FileNotFoundException("Could not locate input file.", filename);
            }

            string buildDefinitionData = File.ReadAllText(filename);

            var buildName = Arguments[TfsUtilityConstants.ArgumentNameBuildDefinitionName];

            var buildInfo = GetBuildInfoByBuildName(buildName);

            if (buildInfo != null)
            {
                UploadUpdatedBuildDefinition(
                    buildInfo.Id, buildInfo.Revision,  buildDefinitionData);
            }
            else
            {
                UploadNewBuild(buildDefinitionData);
            }
        }

        public override void Run()
        {
            Console.WriteLine();

            Upload();

            Console.WriteLine();
        }
    }
}
