﻿using Microsoft.TeamFoundation.Server;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.TeamFoundation.WorkItemTracking.Client;
namespace Benday.TfsUtility
{
    public class ListIterationsCommand : TfsCommandBase
    {
        public ListIterationsCommand(string[] args)
            : base(args)
        {

        }

        protected override List<string> GetRequiredArguments()
        {
            var argumentNames = new List<string>();

            argumentNames.Add(TfsUtilityConstants.ArgumentNameTfsCollection);
            argumentNames.Add(TfsUtilityConstants.ArgumentNameTeamProject);

            return argumentNames;
        }

        protected override void DisplayUsage(StringBuilder builder)
        {
            base.DisplayUsage(builder);

            string usageString =
                String.Format("{0} {1} /collection:collectionurl /project:projectname [/pathsOnly]",
                TfsUtilityConstants.ExeName,
                CommandArgumentName);

            builder.AppendLine(usageString);
        }

        protected override string CommandArgumentName
        {
            get { return TfsUtilityConstants.CommandArgumentNameListIterations; }
        }


        public List<string> GetResult()
        {
            List<string> results = new List<string>();

            Connect();

            var project = FindProjectByName(
                Arguments[TfsUtilityConstants.ArgumentNameTeamProject]);

            if (project == null)
            {
                RaiseExceptionAndDisplayError(true, "TFS project name does not exist.");
            }

            var commonStructureSvc = Tpc.GetService<ICommonStructureService4>();

            AddIterationPathsToResult(commonStructureSvc, project, results);

            return results;
        }

        private void AddIterationPathsToResult(
            ICommonStructureService4 service,
            Project project,
            List<string> results)
        {
            var nodes = project.IterationRootNodes;

            AddIterationPathsToResult(service, project, results, nodes);
        }

        private void AddIterationPathsToResult(
            ICommonStructureService4 service,
            Project project,
            List<string> results,
            NodeCollection nodes)
        {
            NodeInfo nodeInformation = null;

            string nodePathForRetrievingNodeInfo;

            foreach (Node node in nodes)
            {
                if (node.IsIterationNode == true)
                {
                    nodePathForRetrievingNodeInfo =
                        GetPathForRetreivingNodeInfo(project, node);

                    nodeInformation = service.GetNodeFromPath(nodePathForRetrievingNodeInfo);

                    if (nodeInformation != null)
                    {
                        results.Add(ToString(nodeInformation));
                    }

                    if (node.HasChildNodes == true)
                    {
                        AddIterationPathsToResult(
                            service, project, results, node.ChildNodes);
                    }
                }
            }
        }
        private string GetPathForRetreivingNodeInfo(Project project, Node node)
        {
            string nodePathWithoutProjectName =
                GetNodePathWithoutLeadingProjectName(node);

            var returnValue =
                String.Format("{0}\\Iteration{1}", project.Name, nodePathWithoutProjectName);

            return returnValue;
        }

        private string GetNodePathWithoutLeadingProjectName(Node node)
        {
            var fullPath = node.Path;

            var firstBackslash = fullPath.IndexOf("\\");

            if (firstBackslash == -1)
            {
                return fullPath;
            }
            else
            {
                return fullPath.Substring(firstBackslash);
            }
        }

        private string ToString(NodeInfo structure)
        {
            bool pathsOnly =
                Arguments.ContainsKey(TfsUtilityConstants.ArgumentNamePathsOnly);

            if (pathsOnly == true)
            {
                return String.Format("{0}",
                    structure.Path);
            }
            else
            {
                return String.Format("{0} ({1} - {2})",
                    structure.Path,
                    structure.StartDate,
                    structure.FinishDate);
            }
        }

        public override void Run()
        {
            var results = GetResult();
            Console.WriteLine();

            Console.WriteLine("Result count: {0}", results.Count);

            results.ForEach(x => Console.WriteLine(x));
        }
    }
}
