using Benday.TfsUtility.JsonMessageTypes;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using Newtonsoft.Json.Linq;

namespace Benday.TfsUtility
{
    public class ReleaseDefinitionImportCommand : TfsCommandBase
    {
        public ReleaseDefinitionImportCommand(string[] args)
            : base(args)
        {

        }

        protected override List<string> GetRequiredArguments()
        {
            var argumentNames = new List<string>();

            argumentNames.Add(TfsUtilityConstants.ArgumentNameTfsCollection);
            argumentNames.Add(TfsUtilityConstants.ArgumentNameTeamProject);
            argumentNames.Add(TfsUtilityConstants.ArgumentNameReleaseDefinitionName);
            argumentNames.Add(TfsUtilityConstants.ArgumentNameFilename);

            return argumentNames;
        }

        protected override void DisplayUsage(StringBuilder builder)
        {
            base.DisplayUsage(builder);

            string usageString =
                String.Format("{0} {1} /collection:collectionurl /project:projectname /{2}:buildname",
                TfsUtilityConstants.ExeName,
                CommandArgumentName,
                TfsUtilityConstants.ArgumentNameReleaseDefinitionName);

            builder.AppendLine(usageString);
        }

        protected override string CommandArgumentName
        {
            get { return TfsUtilityConstants.CommandArgumentNameReleaseDefinitionImport; }
        }

        private ReleaseDefinitionInfo GetReleaseInfo(string name)
        {
            Console.WriteLine("Checking for existing release named '{0}'...", name);

            string template = "{0}/{1}/_apis/release/definitions?api-version={2}";
            // https://{instance}/defaultcollection/{project}/_apis/release/definitions?api-version={version}[&name={string}][&type={string}]

            string requestUrl = String.Format(
                template,
                Arguments[TfsUtilityConstants.ArgumentNameTfsCollection],
                Arguments[TfsUtilityConstants.ArgumentNameTeamProject],
                "1.0");

            using (WebClient client = new WebClient())
            {
                client.UseDefaultCredentials = true;

                string result = client.DownloadString(requestUrl);

                var results =
                    JsonConvert.DeserializeObject<ReleaseDefinitionInfoResults>(result);

                if (results == null || results.Count == 0)
                {
                    return null;
                }
                else
                {
                    var match = (from temp in results.Values
                                 where String.Equals(
                                     name, 
                                     temp.Name, 
                                     StringComparison.CurrentCultureIgnoreCase) == true
                                 select temp).FirstOrDefault();

                    return match;
                }
            }
        }

        private void UploadUpdatedReleaseDefinition(string id, string revision, string definitionData)
        {
            Console.WriteLine("Modifying existing release (#{0})...", id);

            string template = "{0}/{1}/_apis/release/definitions/{2}?api-version={3}";

            string requestUrl = String.Format(
                template,
                Arguments[TfsUtilityConstants.ArgumentNameTfsCollection],
                Arguments[TfsUtilityConstants.ArgumentNameTeamProject],
                id,
                "1.0");

            using (WebClient client = new WebClient())
            {
                client.UseDefaultCredentials = true;
                client.Headers.Add("Content-Type", "application/json");

                var definition = new ReleaseDefinitionForUploadModified();

                definition.Name = Arguments[TfsUtilityConstants.ArgumentNameReleaseDefinitionName];
                var definitionDataAsJson = JObject.Parse(definitionData);

                definition.Id = id;
                definition.Environments = definitionDataAsJson["environments"] as JArray;
                definition.Artifacts = definitionDataAsJson["artifacts"];
                definition.Variables = definitionDataAsJson["variables"];
                definition.Triggers = definitionDataAsJson["triggers"];

                string definitionUploadAsJson = JsonConvert.SerializeObject(definition, Formatting.Indented);

                string result = client.UploadString(
                    requestUrl, "PUT", definitionUploadAsJson);
            }
        }

        private void RemoveIdProperty(JArray token)
        {
            if (token == null || token.Count == 0)
            {
                return;
            }
            else
            {
                foreach (var arrayItem in token)
                {
                    RemoveIdProperty(arrayItem);
                }
            }
        }

        private void RemoveIdProperty(JToken item)
        {
            JContainer itemAsContainer = item as JContainer;

            if (item != null)
            {
                JProperty itemAsProperty;
                JToken itemToDelete = null;

                foreach (var subitem in itemAsContainer)
                {
                    if (subitem is JProperty)
                    {
                        itemAsProperty = subitem as JProperty;

                        if (itemAsProperty.Name == "id")
                        {
                            itemToDelete = subitem;
                            break;
                        }
                    }
                }

                if (itemToDelete != null)
                {
                    itemToDelete.Remove();
                }
            }
        }

        private void UploadNewRelease(string definitionData)
        {
            Console.WriteLine("Uploading new release...");

            string template = "{0}/{1}/_apis/release/definitions?api-version={2}";

            string requestUrl = String.Format(
                template,
                Arguments[TfsUtilityConstants.ArgumentNameTfsCollection],
                Arguments[TfsUtilityConstants.ArgumentNameTeamProject],
                "1.0");

            using (WebClient client = new WebClient())
            {
                client.UseDefaultCredentials = true;
                client.Headers.Add("Content-Type", "application/json");

                var definition = new ReleaseDefinitionForUploadNew();

                definition.Name = Arguments[TfsUtilityConstants.ArgumentNameBuildDefinitionName];
                var definitionDataAsJson = JObject.Parse(definitionData);

                definition.Environments = definitionDataAsJson["environments"] as JArray;

                RemoveIdProperty(definition.Environments);

                definition.Artifacts = definitionDataAsJson["artifacts"];
                definition.Variables = definitionDataAsJson["variables"];
                definition.Triggers = definitionDataAsJson["triggers"];
                
                string definitionUploadAsJson = JsonConvert.SerializeObject(definition, Formatting.Indented);

                string result = client.UploadString(requestUrl, definitionUploadAsJson);
            }
        }

        public void Upload()
        {
            string filenameArgValue = Arguments[TfsUtilityConstants.ArgumentNameFilename];

            var filename = Path.Combine(Environment.CurrentDirectory, filenameArgValue);

            if (File.Exists(filename) == false)
            {
                throw new FileNotFoundException("Could not locate input file.", filename);
            }

            string definitionData = File.ReadAllText(filename);

            var name = Arguments[TfsUtilityConstants.ArgumentNameReleaseDefinitionName];

            var info = GetReleaseInfo(name);

            if (info != null)
            {
                UploadUpdatedReleaseDefinition(
                    info.Id, info.Revision, definitionData);
            }
            else
            {
                UploadNewRelease(definitionData);
            }
        }

        public override void Run()
        {
            Console.WriteLine();

            Upload();

            Console.WriteLine();
        }
    }
}
